clear all;

% Longueur d'onde (en nanomètres ici)
lambda=600;

% Taille de la fenêtre spatiale
d=70*lambda;

% Largeur caractéristique du faisceau incident
w=10*lambda;

% Angle d'incidence en radians
theta=43*pi/180;

% Position du faisceau par rapport au domaine
% 0.5 = centré
C=0.2;

structure
 
% Caractéristiques de l'image
% Pour l'instant, c'est automatique...

% Nombre de pixels horizontalement
nx=floor(d/30)

% Nombre de pixels pour chaque épaisseur
ny=floor(hauteur/30)

%----------- Ne pas toucher à cette partie ---------------

% Number of modes retained for the description of the field
% so that the last mode has an amplitude < 1e-3
nmod=floor(0.83660*d/w);

hauteur=hauteur/d;
l=lambda/d;
w=w/d;

if pol==0
  f=mu;
else
  f=epsilon;
end
  
k0=2*pi/l;

En=zeros(sum(ny),nx+1);
En1=zeros(sum(ny),nx+1);
En2=zeros(sum(ny),nx+1);

% Total number of layers
g=length(type);

% Amplitude of the different modes
X=exp(-w^2*pi^2*[-nmod:nmod].^2).*exp(-2*i*pi*[-nmod:nmod]*C);   % amplitude de l'onde incidente

% Scattering matrix corresponding to no interface.
T{1}=[0,1;1,0];

for nm=1:2*nmod+1

  alpha= sqrt(epsilon(type(1))*mu(type(1)))*k0*sin(theta)+2*pi*(nm-nmod-1);
  gamma(1)=sqrt(epsilon(type(1))*mu(type(1))*k0^2-alpha^2);  

% Taking the ingoing wave condition into account for the above medium.
  if (real(epsilon(type(1)))<0) & (real(mu(type(1)))<0 )
    gamma(1)=-gamma(1);
  end
    
  for k=1:g-1

% Layer scattering matrix
    t=exp(i*gamma(k)*hauteur(k));    
    T{2*k}=[0,t;t,0];
    gamma(k+1)=sqrt(epsilon(type(k+1))*mu(type(k+1))*k0^2-alpha^2);

% Changing the cut of the square root to achieve perfect stability
    if (imag(gamma(k+1))<0)
      gamma(k+1)=-gamma(k+1);
    end

% Outgoing wave condition for the last medium
    if (real(epsilon(type(g)))<0) & (real(mu(type(g)))<0) & (real(sqrt(epsilon(type(g))*mu(type(g))*k0^2-alpha^2))~=0) gamma(g)=-sqrt(epsilon(type(g))*mu(type(g))*k0^2-alpha^2);
    else
      gamma(g)=sqrt(epsilon(type(g))*mu(type(g))*k0^2-alpha^2);
    end

% Interface scattering matrix
    b1=gamma(k)/f(type(k));
    b2=gamma(k+1)/f(type(k+1));
    T{2*k+1}=[b1-b2,2*b2;2*b1,b2-b1]/(b1+b2);
    end

% Scattering matrix for the last layer
    t=exp(i*gamma(g)*hauteur(g));    
    T{2*g}=[0,t;t,0];

% Once the scattering matrixes have been prepared, now let us combine them.
  
  H{1}=T{2*g};
  A{1}=T{1};
  
  for j=1:size(T,2)-2
    A{j+1}= cascade(A{j},T{j+1}); 	
    H{j+1}= cascade(T{size(T,2)-j},H{j});
  end
  
% And let us compute the intermediate coefficients from the scattering matrixes

  for j=1:size(T,2)-1
    I{j}= intermediaire(A{j},H{size(T,2)-j});
  end

  h=0;
  
  I{2*g}=zeros(2,2);

% Computation of the field in the different layers for one mode (plane wave)

  t=1;
  E=zeros(sum(ny),1);
  for k=1:g
    for m=1:ny(k)
      h=h+hauteur(k)/ny(k);
      	E(t,1)=I{2*k-1}(1,1)*exp(i*gamma(k)*h)+I{2*k}(2,1)*exp(i*gamma(k)*(hauteur(k)-h));
	t=t+1;
    end
    h=0;
  end
  
% For one mode, the image is invariant horizontally
  E=E*exp(i*alpha*[0:nx]./nx);  

% All the images have to be combined using the proper amplitude of each plane wave.
  En=En+X(nm)*E;

end

% -------------- Visualisation --------------------------

V=abs(En).^2;
V=real(En);
colormap(jet);

image_viewer("display %s")
image(V/max(max(V))*128);
